import 'dart:developer' as developer;
import 'dart:io' show Platform;

import 'package:apptics_flutter/rateus/popup_action.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

import 'apptics_flutter_platform_interface.dart';
import 'apptics_flutter_util.dart';
import 'appupdate/apptics_app_update_alerts.dart';
import 'appupdate/appupdate_util.dart';

/// An implementation of [AppticsFlutterPlatform] that uses method channels.

class MethodChannelAppticsFlutter extends AppticsFlutterPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('apptics_flutter');

  @override
  Future<void> setCrashCustomProperty(Map<String, dynamic> properties) async {
    if (Platform.isIOS) {
      methodChannel
          .invokeMethod('setCrashCustomProperty', {"properties": properties});
    } else if (Platform.isAndroid) {
      methodChannel
          .invokeMethod('setCrashCustomProperty', {"properties": properties});
    }
  }

  @override
  Future<void> presentPrivacyReviewPopup() async {
    if (Platform.isIOS) {
      methodChannel.invokeMethod('presentPrivacyReviewPopup');
    } else if (Platform.isAndroid) {
      methodChannel.invokeMethod('presentPrivacyReviewPopup');
    }
  }

  @override
  Future<void> openPrivacySettings() async {
    if (Platform.isIOS) {
      methodChannel.invokeMethod<void>('openPrivacySettings');
    } else if (Platform.isAndroid) {
      methodChannel.invokeMethod<void>('openPrivacySettings');
    }
  }

  @override
  Future<void> setUser(String? userId) async {
    if (Platform.isIOS) {
      methodChannel.invokeMethod<void>('setUser', {"user_id": userId});
    } else if (Platform.isAndroid) {
      methodChannel.invokeMethod<void>('setUser', {"userId": userId});
    }
  }

  @override
  Future<TrackingState?> getTrackingState() async {
    int status =
        await methodChannel.invokeMethod<int>('getTrackingState') ?? -1;
    return TrackingState.getByValue(status);
  }

  @override
  Future<void> setTrackingState(TrackingState status) async {
    if (Platform.isIOS) {
      methodChannel
          .invokeMethod<void>('setTrackingState', {"status": status.value});
    } else if (Platform.isAndroid) {
      methodChannel
          .invokeMethod<void>('setTrackingState', {"status": status.value});
    }
  }

  @override
  Future<bool?> isUserLoggedIn() async {
    if (Platform.isIOS) {
      return await methodChannel.invokeMethod<bool>('isUserLoggedIn');
    } else if (Platform.isAndroid) {
      return await methodChannel.invokeMethod("isUserLoggedIn");
    }
    return null;
  }

  @override
  Future<void> addEvent(
      String? event, String? group, Map<String, dynamic>? properties) async {
    if (event == null || group == null) {
      return;
    }
    if (Platform.isIOS) {
      properties ??= {};
      methodChannel.invokeMethod<void>('addEvent', {"event": event, "group": group, "properties": properties});
    } else if (Platform.isAndroid) {
      methodChannel.invokeMethod<void>('addEvent',
          {"event": event, "group": group, "properties": properties});
    }
  }

  @override
  Future<void> screenAttached(String screen_name) async {
    if (Platform.isIOS) {
      methodChannel
          .invokeMethod<void>('screenAttached', {"screen_name": screen_name});
    } else if (Platform.isAndroid) {
      methodChannel
          .invokeMethod<void>('screenAttached', {"screen_name": screen_name});
    }
  }

  @override
  Future<void> screenDetached(String screen_name) async {
    if (Platform.isIOS) {
      methodChannel
          .invokeMethod<void>('screenDetached', {"screen_name": screen_name});
    } else if (Platform.isAndroid) {
      methodChannel
          .invokeMethod<void>('screenDetached', {"screen_name": screen_name});
    }
  }

  @override
  Future<void> showLastSessionCrashedPopup() async {
    if (Platform.isAndroid) {
      await methodChannel.invokeMethod<String>('showLastSessionCrashedPopup');
    }
  }

  @override
  Future<void> setDefaultLanguage(String lang) async {
    if (Platform.isIOS) {
      methodChannel.invokeMethod<void>('setDefaultLanguage', {"lang": lang});
    } else if (Platform.isAndroid) {
      developer.log('method not available', name: 'Apptics');
    }
  }

  @override
  Future<void> checkAndUpdateAlert(BuildContext context) async {
    // if (true) {
    final response = await methodChannel
        .invokeMapMethod<String, dynamic>("checkAndUpdateAlert");

    ///The value returned from the Native SDK is in the form of a Map<String, dynamic>?.
    if (response != null) {
      if (response['category'] == 1) {
        /*
          mapOf<String, Any>(
            "updateid" to updateData.updateId,
            "currentversion" to updateData.currentVersion,
            "featureTitle" to updateData.featureTitle,
            "features" to updateData.features,
            "remindMeLaterText" to updateData.remindMeLaterText,
            "updateNowText" to updateData.updateNowText,
            "neverAgainText" to updateData.neverAgainText,
            "option" to updateData.option,
            "reminderDays" to updateData.reminderDays,
            "forceInDays" to updateData.forceInDays,
            "alertType" to updateData.alertType,
            "customStoreUrl" to updateData.customStoreUrl,
            "category" to 1
        )*/

        final map = Map<String, dynamic>.from(response.cast<String, dynamic>());
        final updateData = AppUpdateData.fromJson(map);
        Future.delayed(
            Duration.zero,
            () => AppticsInAppUpdateAlert.show(
                context: context,
                appUpdateData: updateData,
                onClickUpdate: onClickUpdate,
                onClickReminder: onClickReminder,
                onClickIgnore: onClickIgnore));
        onSendImpressionStatus(updateData.updateId);
      } else if (response['category'] == 2) {
        /*
          mapOf<String, Any>(
                "title" to appticsAppUpdateNotSupported.title,
                "description" to appticsAppUpdateNotSupported.description,
                "continueBtTxt" to appticsAppUpdateNotSupported.continueBtTxt,
                "alertType" to appticsAppUpdateNotSupported.alertType,
                "updateId" to appticsAppUpdateNotSupported.updateId,
                "category" to 2
            )
            */

        final map = Map<String, dynamic>.from(response.cast<String, dynamic>());
        final updateData = AppticsAppUpdateNotSupported.fromJson(map);
        Future.delayed(
            Duration.zero,
            () => AppticsInAppUpdateAlert.showNonSupportAlert(
                context: context,
                appUpdateData: updateData,
                onClick: onClickNonSupportAlert));
        onSendImpressionStatus(updateData.updateId);
      }
    }
    // }
  }

  @override
  Future<void> onClickUpdate() async {
    methodChannel.invokeMethod('onClickUpdate');
  }

  @override
  Future<void> onClickReminder() async {
    methodChannel.invokeMethod('onClickReminder');
  }

  @override
  Future<void> onClickIgnore() async {
    methodChannel.invokeMethod('onClickIgnore');
  }

  @override
  Future<void> onClickNonSupportAlert() async {
    methodChannel.invokeMethod('onClickNonSupportAlert');
  }

  @override
  Future<void> onSendImpressionStatus(String updateId) async {
    methodChannel
        .invokeMethod('onSendImpressionStatus', {'updateId': updateId});
  }

  @override
  Future<Map<String, dynamic>?> getInAppUpdateData() async {
    return await methodChannel
        .invokeMapMethod<String, dynamic>("checkAndUpdateAlert");
  }

  // In-app Ratings

  @override
  Future<int?> checkForRatingPop() async {
    return await methodChannel.invokeMethod<int>("checkForRatingPop");
  }

  @override
  Future<bool?> isAppticsFeedbackModuleAvailable() async {
    return await methodChannel
        .invokeMethod<bool>("isAppticsFeedbackModuleAvailable");
  }

  @override
  Future<void> openFeedbackScreen() async {
    return await methodChannel.invokeMethod("openFeedbackScreen");
  }

  int _daysBeforeShowingPopupAgain = 10;

  int _maxTimesToShowPopup = 3;

  bool _disableAutoPromptOnFulFillingCriteria = false;

  bool _showStoreAlertOnFulFillingCriteria = false;

  bool _disableIfNotInstalledFromPlayStore = false;

  @override
  int get daysBeforeShowingPopupAgain {
    return _daysBeforeShowingPopupAgain;
  }

  @override
  int get maxTimesToShowPopup {
    return _maxTimesToShowPopup;
  }

  @override
  bool get disableAutoPromptOnFulFillingCriteria {
    return _disableAutoPromptOnFulFillingCriteria;
  }

  @override
  bool get showStoreAlertOnFulFillingCriteria {
    return _showStoreAlertOnFulFillingCriteria;
  }

  @override
  bool get disableIfNotInstalledFromPlayStore {
    return _disableIfNotInstalledFromPlayStore;
  }

  @override
  Future<void> setDaysBeforeShowingPopupAgain(int dayCount) async {
    try {
      final day = await methodChannel.invokeMethod<int>(
          "setDaysBeforeShowingPopupAgain", {'dayCount': dayCount});
      if (day != null) _daysBeforeShowingPopupAgain = day;
    } on PlatformException catch (e, s) {
      print(s);
    }
  }

  @override
  Future<void> setMaxTimesToShowPopup(int maxTime) async {
    try {
      final count = await methodChannel
          .invokeMethod<int>("setMaxTimesToShowPopup", {'maxTime': maxTime});
      if (count != null) _maxTimesToShowPopup = count;
    } on PlatformException catch (e, s) {
      print(s);
    }
  }

  @override
  Future<void> setDisableAutoPromptOnFulFillingCriteria(bool isDisable) async {
    try {
      final checkAutoPrompt = await methodChannel.invokeMethod<bool>(
          "setDisableAutoPromptOnFulFillingCriteria", {'isDisable': isDisable});
      if (checkAutoPrompt != null)
        _disableAutoPromptOnFulFillingCriteria = isDisable;
    } on PlatformException catch (e, s) {
      print(s);
    }
  }

  @override
  Future<void> setShowStoreAlertOnFulFillingCriteria(bool isShow) async {
    try {
      final checkAndroidPlayCoreAlert = await methodChannel.invokeMethod<bool>(
          "setShowStoreAlertOnFulFillingCriteria", {'isShow': isShow});
      if (checkAndroidPlayCoreAlert != null)
        _showStoreAlertOnFulFillingCriteria = isShow;
    } on PlatformException catch (e, s) {
      print(s);
    }
  }

  @override
  Future<void> setDisableIfNotInstalledFromPlayStore(
      bool ifNotInstalledFromPlayStore) async {
    try {
      final checkNotInstalledPlayStore = await methodChannel.invokeMethod<bool>(
          "setDisableIfNotInstalledFromPlayStore",
          {'ifNotInstalledFromPlayStore': ifNotInstalledFromPlayStore});
      if (checkNotInstalledPlayStore != null)
        _disableIfNotInstalledFromPlayStore = ifNotInstalledFromPlayStore;
    } on PlatformException catch (e, s) {
      print(s);
    }
  }

  @override
  Future<void> sentRatingStats(int criteriaId, PopupAction popupAction) async {
    try {
      await methodChannel.invokeMethod("sentRatingStats",
          {'criteriaId': criteriaId, 'popupAction': popupAction.value});
    } on PlatformException catch (e, s) {
      print(s);
    }
  }

  @override
  Future<void> updateRatingShown() async {
    try {
      await methodChannel.invokeMethod("updateRatingShown");
    } on PlatformException catch (e, s) {
      print(s);
    }
  }

  @override
  Future<void> openPlayStore() async {
    try {
      await methodChannel.invokeMethod("openPlayStore");
    } on PlatformException catch (e, s) {
      print(s);
    }
  }

  @override
  Future<void> hardReset() async {
    try {
      return await methodChannel.invokeMethod("hardReset");
    } on PlatformException catch (e, s) {
      print(s);
    }
  }

  @override
  Future<String?> getStringValue(
    String key, {
    bool coldFetch = false,
    bool fallbackWithOfflineValue = false,
  }) async {
    try {
      return await methodChannel.invokeMethod<String>("getStringValue", {
        'key': key,
        'coldFetch': coldFetch,
        'fallbackWithOfflineValue': fallbackWithOfflineValue
      });
    } on PlatformException catch (e, s) {
      print(s);
      return null;
    }
  }

  @override
  Future<void> setCustomConditionValue(String key, String value) async {
    try {
      return await methodChannel.invokeMethod(
          "setCustomConditionValue", {'key': key, 'value': value});
    } on PlatformException catch (e, s) {
      print(s);
    }
  }
}
