import 'dart:developer' as developer;
import 'dart:io';

import 'package:apptics_flutter/rateus/popup_action.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import '../apptics_flutter_platform_interface.dart';

class AppticsInAppRating {
  static AppticsInAppRating? _instance;

  static AppticsInAppRating get instance {
    _instance ??= AppticsInAppRating();
    return _instance!;
  }

  Future _showAlertDialog(
    BuildContext context,
    String title,
    String message, {
    bool isFeedbackEnabled = false,
    required void Function() onClickOpenStore,
    required void Function() onClickSendFeedback,
    required void Function() onClickLater,
  }) async {
    final isDarkMode =
        MediaQuery.of(context).platformBrightness == Brightness.dark;

    final isDarkColorText = isDarkMode
        ? const TextStyle(color: Colors.white)
        : const TextStyle(color: Colors.black);
    final isDarkBackgroundColor = isDarkMode ? Colors.black : Colors.white;
    final textButtonStyle = TextButton.styleFrom(
      textStyle: Theme.of(context).textTheme.labelLarge,
    );
    if (Platform.isIOS) {
      List<Widget> widgetList = <Widget>[];

      widgetList.add(CupertinoDialogAction(
          child: TextButton(
        child: const Text("RATE IN APP STORE"),
        onPressed: () {
          onClickOpenStore();
          Navigator.of(context).pop(); // Close the dialog
        },
      )));
      if (isFeedbackEnabled) {
        widgetList.add(CupertinoDialogAction(
            child: TextButton(
          child: const Text("SEND FEEDBACK"),
          onPressed: () {
            onClickSendFeedback();
            Navigator.of(context).pop(); // Close the dialog
          },
        )));
      }
      widgetList.add(CupertinoDialogAction(
          child: TextButton(
        child: const Text("LATER"),
        onPressed: () {
          onClickLater();
          Navigator.of(context).pop(); // Close the dialog
        },
      )));

      showCupertinoDialog(
          context: context,
          builder: (context) {
            return CupertinoAlertDialog(
              title: Text(title, style: isDarkColorText),
              content: Text(message, style: isDarkColorText),
              actions: widgetList,
            );
          });
    } else {
      List<Widget> listWidget = <Widget>[];

      listWidget.add(TextButton(
        style: textButtonStyle,
        onPressed: () {
          onClickOpenStore();
          Navigator.of(context).pop(); // Close the dialog
        },
        child: const Text("RATE IN PLAY STORE"),
      ));
      if (isFeedbackEnabled) {
        listWidget.add(TextButton(
          style: textButtonStyle,
          onPressed: () {
            onClickSendFeedback();
            Navigator.of(context).pop(); // Close the dialog
          },
          child: const Text("SEND FEEDBACK"),
        ));
      }
      listWidget.add(TextButton(
        style: textButtonStyle,
        onPressed: () {
          onClickLater();
          Navigator.of(context).pop(); // Close the dialog
        },
        child: const Text("LATER"),
      ));

      showDialog(
        context: context,
        builder: (context) {
          return AlertDialog(
            title: Text(title, style: isDarkColorText),
            content: Text(message),
            contentTextStyle: isDarkColorText,
            backgroundColor: isDarkBackgroundColor,
            alignment: Alignment.center,
            actions: listWidget,
          );
        },
      );
    }
  }

  /// Check for In-app Ratings.
  /// @params context
  /// @param isFeedbackEnabled [bool] - Set to true to activate the "Send Feedback" option.
  ///
  Future<void> checkForRatingPop(BuildContext context, {bool isFeedbackEnabled = false}) async {
      int? criteriaId =
          await AppticsFlutterPlatform.instance.checkForRatingPop();

      if (criteriaId == null || criteriaId == 0 || !context.mounted) return;

      updateRatingShown();
      Future.delayed(
          Duration.zero,
          () => _showAlertDialog(
                  context, "Rate Our App", "Do you enjoy using this app?",
                  isFeedbackEnabled: isFeedbackEnabled, onClickOpenStore: () {
                openPlayStore();
                sentStats(criteriaId, PopupAction.RATE_IN_STORE_CLICKED);
              }, onClickSendFeedback: () {
                openFeedback();
                sentStats(criteriaId, PopupAction.SEND_FEEDBACK_CLICKED);
              }, onClickLater: () {
                sentStats(criteriaId, PopupAction.LATER_CLICKED);
              }));    
  }

  /// Build Your own UI for using
  Future<int?> getCriteriaId() async {
    return await AppticsFlutterPlatform.instance.checkForRatingPop();
  }

  Future<bool?> isAppticsFeedbackModuleAvailable() {
    return AppticsFlutterPlatform.instance.isAppticsFeedbackModuleAvailable();
  }

  Future<void> openFeedback() async {
    return AppticsFlutterPlatform.instance.openFeedbackScreen();
  }

  Future<void> openPlayStore() async {
    return AppticsFlutterPlatform.instance.openPlayStore();
  }

//What is the purpose of this method?
  /// Applicable exclusively to the Android platform.
  int get daysBeforeShowingPopupAgain {
    return AppticsFlutterPlatform.instance.daysBeforeShowingPopupAgain;
  }

  /// Number of times to show in-app ratings pop-up.
  /// Applicable exclusively to the Android platform.
  int get maxTimesToShowPopup {
    return AppticsFlutterPlatform.instance.maxTimesToShowPopup;
  }

  /// Applicable exclusively to the Android platform.
  bool get disableAutoPromptOnFulFillingCriteria {
    return AppticsFlutterPlatform
        .instance.disableAutoPromptOnFulFillingCriteria;
  }

  /// Applicable exclusively to the Android platform.
  bool get showStoreAlertOnFulFillingCriteria {
    return AppticsFlutterPlatform.instance.showStoreAlertOnFulFillingCriteria;
  }

  /// Applicable exclusively to the Android platform.
  bool get disableIfNotInstalledFromPlayStore {
    return AppticsFlutterPlatform.instance.disableIfNotInstalledFromPlayStore;
  }

  ///Set number of days before showing ratings pop-up again once user cancels/ignores the pop-up.
  ///Applicable exclusively to the Android platform.
  Future<void> setDaysBeforeShowingPopupAgain(int dayCount) async {
    if (Platform.isAndroid) {
      AppticsFlutterPlatform.instance.setDaysBeforeShowingPopupAgain(dayCount);
    } else {
      developer.log('method not available in iOS.', name: 'Apptics');
    }
  }

  /// Number of times to show in-app ratings pop-up. default value 3.
  /// Applicable exclusively to the Android platform.
  Future<void> setMaxTimesToShowPopup(int maxTime) async {
    if (Platform.isAndroid) {
      AppticsFlutterPlatform.instance.setMaxTimesToShowPopup(maxTime);
    } else {
      developer.log('method not available in iOS.', name: 'Apptics');
    }
  }

  /// Disable showing in-app ratings pop-up automatically on satisfying criterion.
  /// Applicable exclusively to the Android platform.
  Future<void> setDisableAutoPromptOnFulFillingCriteria(bool isDisable) async {
    if (Platform.isAndroid) {
      AppticsFlutterPlatform.instance.setDisableAutoPromptOnFulFillingCriteria(isDisable);
    } else {
      developer.log('method not available in iOS.', name: 'Apptics');
    }
  }

  /// To invoke play-core in-app ratings SDK on satisfying criterion.
  /// Applicable exclusively to the Android platform.
  Future<void> setShowStoreAlertOnFulFillingCriteria(bool isShow) async {
    AppticsFlutterPlatform.instance.setShowStoreAlertOnFulFillingCriteria(isShow);
  }

  /// Flag to disable ratings if the app is not installed from playStore.
  /// Applicable exclusively to the Android platform.
  Future<void> setDisableIfNotInstalledFromPlayStore(
      bool ifNotInstalledFromPlayStore) async {
    if (Platform.isAndroid) {
      AppticsFlutterPlatform.instance
        .setDisableIfNotInstalledFromPlayStore(ifNotInstalledFromPlayStore);
    } else {
      developer.log('method not available in iOS.', name: 'Apptics');
    }    
  }

  /// Update in-app ratings engagement stats to Apptics. Used in-case of building your own UI for in-app ratings.
  /// @param criteriaId Long value provided in customUiCallback.
  /// @param popupAction [PopupAction] enum to represent engagement.
  Future<void> sentStats(int criteriaId, PopupAction popupAction) async {
    await AppticsFlutterPlatform.instance
        .sentRatingStats(criteriaId, popupAction);
  }

  /// Method for marking the rating popup.
  Future<void> updateRatingShown() async {
    await AppticsFlutterPlatform.instance
        .updateRatingShown();
  }

}
